// Copyright (c) 1994, 1995 James Clark
// See the file COPYING for copying permission.

#ifndef EntityToolkit_INCLUDED
#define EntityToolkit_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

#include "CodingSystem.H"
#include "CharsetInfo.H"
#include "GrowableVector.H"
#include "MessageReporter.H"
#include "Boolean.H"
#include "ExtendEntityManager.H"
#include "Location.H"

class PosixStorageManager;
class PosixFdStorageManager;
class URLStorageManager;
class InputSource;
class streambuf;

class EntityToolkit : public MessageReporter {
public:
  EntityToolkit();
  virtual ~EntityToolkit();
  void setErrorStream(streambuf *);
  void addSearchDir(const char *);
  CString messageText(const char *domain, const char *id);
  const CodingSystem *codingSystem() const;
  ResourcePointer<ExtendEntityManager> &entityManager();
  Boolean makeSystemId(int nFiles, char *const *files, CString &result);
  void initEntityManager();
  CString convertInput(const char *s);
  InputSource *open(const CString &, Boolean mayRewind = 0);
  static const CodingSystem *lookupCodingSystem(const char *);
protected:
  CharsetInfo systemCharset_;
private:
  EntityToolkit(const EntityToolkit &); // undefined
  void operator=(const EntityToolkit &); // undefined
  PosixStorageManager *storageManager_;
  PosixFdStorageManager *fdStorageManager_;
  URLStorageManager *urlStorageManager_;
  ResourcePointer<ExtendEntityManager> entityManager_;
  const CodingSystem *codingSystem_;
  GrowableVector<const char *> searchDirs_;
  IosOutputCharStream errorStream_;
};

inline
const CodingSystem *EntityToolkit::codingSystem() const
{
  return codingSystem_;
}

inline
ResourcePointer<ExtendEntityManager> &EntityToolkit::entityManager()
{
  return entityManager_;
}

inline
InputSource *EntityToolkit::open(const CString &sysid, Boolean mayRewind)
{
  return entityManager_->open(sysid,
			      systemCharset_,
			      new InputSourceOrigin,
			      mayRewind,
			      *this);
}

#endif /* not EntityToolkit_INCLUDED */
