// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef OutputCharStream_INCLUDED
#define OutputCharStream_INCLUDED 1

#include "types.H"
#include <stddef.h>
#include "CString.H"
#include "Owner.H"
#include "CodingSystem.H"

class streambuf;

class OutputCharStream {
public:
  typedef void (*Escaper)(OutputCharStream &, Char);
  OutputCharStream();
  virtual ~OutputCharStream();
  OutputCharStream &put(Char);
  OutputCharStream &write(const Char *, size_t);
  virtual void flush() = 0;
  void setEscaper(Escaper);

  OutputCharStream &operator<<(char);
  OutputCharStream &operator<<(const char *);
  OutputCharStream &operator<<(const CString &);
  OutputCharStream &operator<<(unsigned long);
  OutputCharStream &operator<<(int);
private:
  OutputCharStream(const OutputCharStream &);	// undefined
  void operator=(const OutputCharStream &);	// undefined

  virtual void flushBuf(Char) = 0;
  Escaper escaper_;
protected:
  void escape(OutputCharStream &, Char c);
  Char *ptr_;
  Char *end_;
};

class IosOutputCharStream : public OutputCharStream,
                            private Encoder::Handler {
public:
  IosOutputCharStream();
  // the streambuf will not be deleted
  IosOutputCharStream(streambuf *, const OutputCodingSystem *);
  ~IosOutputCharStream();
  void open(streambuf *, const OutputCodingSystem *);
  void flush();
private:
  IosOutputCharStream(const IosOutputCharStream &); // undefined
  void operator=(const IosOutputCharStream &);	    // undefined
  IosOutputCharStream(streambuf *, Encoder *);
  void allocBuf(int bytesPerChar);
  void flushBuf(Char);
  void handleUnencodable(Char c, streambuf *);
  Char *buf_;
  streambuf *byteStream_;
  Encoder *encoder_;
  Owner<Encoder> ownedEncoder_;
};

class StrOutputCharStream : public OutputCharStream {
public:
  StrOutputCharStream();
  void extractString(CString &);
  void flush();
private:
  void flushBuf(Char);
  void sync(size_t);
  StrOutputCharStream(const StrOutputCharStream &); // undefined
  void operator=(const StrOutputCharStream &);	    // undefined
  CString str_;
};

inline
OutputCharStream &OutputCharStream::put(Char c)
{
  if (ptr_ < end_)
    *ptr_++ = c;
  else
    flushBuf(c);
  return *this;
}

inline
OutputCharStream &OutputCharStream::operator<<(char c)
{
  return put(Char(c));
}

inline
OutputCharStream &OutputCharStream::operator<<(const CString &str)
{
  return write(str.pointer(), str.length());
}

inline
void OutputCharStream::setEscaper(Escaper f)
{
  escaper_ = f;
}

inline
void OutputCharStream::escape(OutputCharStream &s, Char c)
{
  if (escaper_)
    (*escaper_)(s, c);
}

#endif /* not OutputCharStream_INCLUDED */
