// Copyright (c) 1995 James Clark
// See the file COPYING for copying permission.

#ifndef EntityDecl_INCLUDED
#define EntityDecl_INCLUDED 1

#ifdef __GNUG__
#pragma interface
#endif

#include "NamedResource.H"
#include "ResourcePointer.H"
#include "StringResource.H"
#include "Location.H"
#include "types.H"

class EntityDecl : public NamedResource {
public:
  enum DeclType { generalEntity, parameterEntity, doctype, linktype,
		  notation };
  enum DataType { sgmlText, pi, cdata, sdata, ndata, subdoc };
  EntityDecl(const CString &, DeclType declType, DataType dataType,
	     const Location &defLocation);
  DataType dataType() const;
  DeclType declType() const;
  const Location &defLocation() const;
  Boolean declInDtdIsBase() const;
  const CString *declInDtdNamePointer() const;
  const CString *declInLpdNamePointer() const;
  void setDeclIn(const ConstResourcePointer<StringResource<Char> > &dtdName,
		 Boolean dtdIsBase,
		 const ConstResourcePointer<StringResource<Char> > &lpdName);
  void setDeclIn(const ConstResourcePointer<StringResource<Char> > &dtdName,
		 Boolean dtdIsBase);
  void setDefLocation(const Location &);
  virtual const CString *systemIdPointer() const;
  virtual const CString *publicIdPointer() const;
private:
  DeclType declType_;
  DataType dataType_;
  PackedBoolean dtdIsBase_;
  Location defLocation_;
  ConstResourcePointer<StringResource<Char> > dtdName_;
  ConstResourcePointer<StringResource<Char> > lpdName_;
};

inline
const Location &EntityDecl::defLocation() const
{
  return defLocation_;
}

inline
EntityDecl::DeclType EntityDecl::declType() const
{
  return declType_;
}

inline
EntityDecl::DataType EntityDecl::dataType() const
{
  return dataType_;
}

inline
const CString *EntityDecl::declInDtdNamePointer() const
{
  return dtdName_.pointer();
}

inline
const CString *EntityDecl::declInLpdNamePointer() const
{
  return lpdName_.pointer();
}

inline
Boolean EntityDecl::declInDtdIsBase() const
{
  return dtdIsBase_;
}

inline
void EntityDecl::setDefLocation(const Location &loc)
{
  defLocation_ = loc;
}

#endif /* not EntityDecl_INCLUDED */
