// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef CharsetDecl_INCLUDED
#define CharsetDecl_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

#include "types.H"
#include "GrowableVectorD.H"
#include "ExternalId.H"
#include "ISet.H"
#include "Boolean.H"

class CharsetDeclRange {
public:
  enum Type {
    number,
    string,
    unused
    };
  CharsetDeclRange();
  CharsetDeclRange(WideChar, Number, WideChar);
  CharsetDeclRange(WideChar, Number);
  CharsetDeclRange(WideChar, Number, const CString &);
  void moveTo(CharsetDeclRange &to);
  void rangeDeclared(WideChar min, Number count,
		     ISet<WideChar> &declared) const;
  void usedSet(ISet<Char> &) const;
  Boolean getCharInfo(WideChar fromChar,
		      CharsetDeclRange::Type &type,
		      Number &n,
		      CString &str,
		      Number &count) const;
  void stringToChar(const CString &str, ISet<WideChar> &to) const;
  void numberToChar(Number n, ISet<WideChar> &to, Number &count) const;
private:
  WideChar descMin_;
  Number count_;
  WideChar baseMin_;
  Type type_;
  CString str_;
};

class CharsetDeclSection {
public:
  CharsetDeclSection();
  CharsetDeclSection(PublicId &);
  void addRange(CharsetDeclRange &);
  void moveTo(CharsetDeclSection &);
  void rangeDeclared(WideChar min, Number count,
		     ISet<WideChar> &declared) const;
  void usedSet(ISet<Char> &) const;
  Boolean getCharInfo(WideChar fromChar,
		      const PublicId *&id,
		      CharsetDeclRange::Type &type,
		      Number &n,
		      CString &str,
		      Number &cout) const;
  void stringToChar(const CString &str, ISet<WideChar> &to) const;
  void numberToChar(const PublicId *id, Number n,
		    ISet<WideChar> &to, Number &count) const;
private:
  CharsetDeclSection(const CharsetDeclSection &); // undefined
  void operator=(const CharsetDeclSection &);	  // undefined
  PublicId baseset_;
  GrowableVectorD<CharsetDeclRange> ranges_;
};

class CharsetDecl {
public:
  CharsetDecl();
  void addSection(PublicId &);
  void moveTo(CharsetDecl &);
  void clear();
  void usedSet(ISet<Char> &) const;
  void declaredSet(ISet<WideChar> &set) const;
  Boolean charDeclared(WideChar) const;
  void rangeDeclared(WideChar min, Number count,
		     ISet<WideChar> &declared) const;
  void addRange(WideChar, Number, WideChar);
  void addRange(WideChar, Number);
  void addRange(WideChar, Number, const CString &);
  Boolean getCharInfo(WideChar fromChar,
		      const PublicId *&id,
		      CharsetDeclRange::Type &type,
		      Number &n,
		      CString &str) const;
  Boolean getCharInfo(WideChar fromChar,
		      const PublicId *&id,
		      CharsetDeclRange::Type &type,
		      Number &n,
		      CString &str,
		      Number &count) const;
  void stringToChar(const CString &str, ISet<WideChar> &to) const;
  void numberToChar(const PublicId *id, Number n,
		    ISet<WideChar> &to, Number &count) const;
  void numberToChar(const PublicId *id, Number n, ISet<WideChar> &to) const;
private:
  CharsetDecl(const CharsetDecl &); // undefined
  void operator=(const CharsetDecl &); // undefined
  GrowableVectorD<CharsetDeclSection> sections_;
  ISet<WideChar> declaredSet_;
};

inline
Boolean CharsetDecl::getCharInfo(WideChar fromChar,
				 const PublicId *&id,
				 CharsetDeclRange::Type &type,
				 Number &n,
				 CString &str) const
{
  Number tem;
  return getCharInfo(fromChar, id, type, n, str, tem);
}

inline
void CharsetDecl::numberToChar(const PublicId *id, Number n,
			       ISet<WideChar> &to) const
{
  Number tem;
  numberToChar(id, n, to, tem);
}

inline
void CharsetDecl::declaredSet(ISet<WideChar> &set) const
{
  set = declaredSet_;
}

inline
Boolean CharsetDecl::charDeclared(WideChar c) const
{
  return declaredSet_.contains(c);
}

#endif /* not CharsetDecl_INCLUDED */
