// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif
#include "config.h"
#include "CharsetRegistry.H"
#include "ExternalId.H"
#include "CharsetInfo.H"
#include "UnivCharsetDesc.H"
#include "CString.H"
#include "types.H"
#include "macros.H"

static UnivCharsetDesc::Range iso646_irv[] = {
  { 0, 128, 0 }
};

static UnivCharsetDesc::Range iso646_C0[] = {
  { 0, 32, 0 },
  { 127, 1, 127 },
};

static struct {
  const char *sequence;
  const UnivCharsetDesc::Range *ranges;
  size_t nRanges;
} table[] = {
  { "ESC 2/5 4/0", iso646_irv, SIZEOF(iso646_irv) },
  { "ESC 2/8 4/0", iso646_irv, SIZEOF(iso646_irv) },
  { "ESC 2/8 4/2", iso646_irv, SIZEOF(iso646_irv) }, // ASCII
  { "ESC 2/1 4/0", iso646_C0, SIZEOF(iso646_C0) },
};

Boolean CharsetRegistry::findCharset(const PublicId &id,
				     const CharsetInfo &charset,
				     UnivCharsetDesc &desc)
{
  PublicId::OwnerType ownerType;
  if (!id.getOwnerType(ownerType) || ownerType != PublicId::ISO)
    return 0;
  CString sequence;
  if (!id.getDesignatingSequence(sequence))
    return 0;
  // Canonicalize the escape sequence by mapping esc -> ESC,
  // removing leading zeros from escape sequences, and removing
  // initial spaces.
  CString s;
  size_t i;
  for (i = 0; i < sequence.length(); i++) {
    Char c = sequence[i];
    if (c == charset.execToDesc('e'))
      s += charset.execToDesc('E');
    else if (c == charset.execToDesc('s'))
      s += charset.execToDesc('S');
    else if (c == charset.execToDesc('c'))
      s += charset.execToDesc('C');
    else if (charset.digitWeight(c) >= 0
	     && s.length() > 0
	     && s[s.length() - 1] == charset.execToDesc('0')
	     && (s.length() == 1
		 || charset.digitWeight(s[s.length() - 2]) >= 0))
      s[s.length() - 1] = c;
    else if (c != charset.execToDesc(' ') || s.length() > 0)
      s += c;
  }
  for (i = 0; i < SIZEOF(table); i++)
    if (s == charset.execToDesc(table[i].sequence)) {
      desc.set(table[i].ranges, table[i].nRanges);
      return 1;
    }
  return 0;
}
