// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif
#include "config.h"
#include "Dtd.H"
#include "setAll.H"
#include "Syntax.H"

Dtd::Dtd(const CString &name, Boolean isBase)
: name_(new StringResource<Char>(name)),
  nCurrentAttribute_(0),
  nElementDefinition_(0),
  nAttributeDefinitionList_(0),
  isBase_(isBase)
{
  documentElementType_ = new ElementType(name, nElementTypeIndex());
  insertElementType(documentElementType_);
}

Boolean Dtd::shortrefIndex(const CString &str, const Syntax &syntax,
			   size_t &index)
{
  const int *indexP = shortrefTable_.lookup(str);
  if (indexP) {
    index = *indexP;
    return 1;
  }
  if (!syntax.isValidShortref(str))
    return 0;
  shortrefTable_.insert(str, int(shortrefs_.length()));
  index = shortrefs_.length();
  shortrefs_.grow() = str;
  return 1;
}

void Dtd::addNeededShortref(const CString &str)
{
  if (!shortrefTable_.lookup(str)) {
    shortrefTable_.insert(str, shortrefs_.length());
    shortrefs_.grow() = str;
  }
}

void Dtd::setDefaultEntity(const ResourcePointer<Entity> &entity,
			   ParserState &parser)
{
  defaultEntity_ = entity;
  
  // If the new default entity was defined in a DTD, then
  // any defaulted entities must have come from an LPD
  // on the first pass, in which case we shouldn't replace them.
  if (entity->declInLpdNamePointer() != 0) {
    NamedResourceTable<Entity> tem;
    
    EntityIter iter(defaultedEntityTable_);
    for (;;) {
      ResourcePointer<Entity> old(iter.next());
      if (old.isNull())
	break;
      ResourcePointer<Entity> e(defaultEntity_->copy());
      e->setName(old->name());
      e->generateSystemId(parser);
      tem.insert(e);
    }
    tem.moveTo(defaultedEntityTable_);
  }
}
