// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif
#include "config.h"
#include "Notation.H"
#include "ParserState.H"
#include "Sd.H"
#include "Syntax.H"
#include "MessageArg.H"
#include "ParserMessages.H"

Notation::Notation(const CString &name,
		   const ConstResourcePointer<StringResource<Char> > &dtdName,
		   Boolean dtdIsBase)
: EntityDecl(name, notation, ndata, Location()), defined_(0)
{
  setDeclIn(dtdName, dtdIsBase);
}

void Notation::setExternalId(ExternalId &id, const Location &defLocation)
{
  id.moveTo(externalId_);
  defined_ = 1;
  setDefLocation(defLocation);
}

void Notation::generateSystemId(ParserState &parser)
{
  if (parser.options().generateNotationSystemId) {
    CString str;
    if (parser.entityCatalog().lookup(*this,
				      parser.syntax(),
				      parser.sd().docCharset(),
				      parser.messenger(),
				      str))
      externalId_.setEffectiveSystem(str);
    else
      parser.message(ParserMessages::cannotGenerateSystemIdNotation,
		     StringMessageArg(name()));
  }
}

const CString *Notation::systemIdPointer() const
{
  return externalId_.systemIdString();
}

const CString *Notation::publicIdPointer() const
{
  return externalId_.publicIdString();
}

