// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif

#include "config.h"
#include "Parser.H"
#include "ParserMessages.H"
#include "constant.H"

Parser::Parser(const CString &document,
	       const ResourcePointer<InputSourceOrigin> &origin,
	       const ConstResourcePointer<EntityManager> &em,
	       const ConstResourcePointer<EntityCatalog> &ec,
	       const UnivCharsetDesc &initialCharset,
	       const ParserOptions &opt)
: ParserState(em, ec, opt)
{
  Sd *sd = new Sd();
  sd->setDocCharsetDesc(initialCharset);
  sd->setBooleanFeature(Sd::fDATATAG, opt.datatag);
  sd->setBooleanFeature(Sd::fOMITTAG, opt.omittag);
  sd->setBooleanFeature(Sd::fRANK, opt.rank);
  sd->setBooleanFeature(Sd::fSHORTTAG, opt.shorttag);
  sd->setNumberFeature(Sd::fSIMPLE, opt.linkSimple);
  sd->setBooleanFeature(Sd::fIMPLICIT, opt.linkImplicit);
  sd->setNumberFeature(Sd::fEXPLICIT, opt.linkExplicit);
  sd->setNumberFeature(Sd::fCONCUR, opt.concur);
  sd->setNumberFeature(Sd::fSUBDOC, opt.subdoc);
  sd->setBooleanFeature(Sd::fFORMAL, opt.formal);
  PublicId publicId;
  CharsetDecl docCharsetDecl;
  docCharsetDecl.addSection(publicId);
  docCharsetDecl.addRange(0, charMax > 99999999 ? 99999999 : charMax + 1, 0);
  sd->setDocCharsetDecl(docCharsetDecl);
  setSd(sd);
  pushInput(em->open(document,
		     sd->docCharset(),
		     origin.pointer(),
		     1,
		     messenger()));
  setDoFunction(&Parser::doInit);
}

Parser::Parser(const Parser &parent,
	       const SubdocEntity &subdoc,
	       const ResourcePointer<InputSourceOrigin> &origin,
	       const ParserSubdocOptions &options)
: ParserState(parent, options)
{
  if (subdoc.externalId().effectiveSystemId().length())
    pushInput(entityManager()
	      .open(subdoc.externalId().effectiveSystemId(),
		    sd().docCharset(),
		    origin.pointer(),
		    1,
		    messenger()));
  setDoFunction(&Parser::doProlog);
  compilePrologModes();
}

void Parser::giveUp()
{
  if (subdocLevel() > 0)	// FIXME might be subdoc if level == 0
    message(ParserMessages::subdocGiveUp);
  else
    message(ParserMessages::giveUp);
  allDone();
}

Event *Parser::nextEvent()
{
  while (eventQueueEmpty()) {
    if (!haveDoFunction())
      return 0;
    // Use temporary to work round cfront limitation
    DoFunction f = doFunction();
    (this->*f)();
  }
  return eventQueueGet();
}

void Parser::parseAll(EventHandler &handler)
{
  while (!eventQueueEmpty())
    eventQueueGet()->handle(handler);
  // FIXME catch exceptions and reset handler.
  setHandler(&handler);
  while (haveDoFunction()) {
    // Use temporary to work round cfront limitation
    DoFunction f = doFunction();
    (this->*f)();
  }
  unsetHandler();
}
