// Copyright (c) 1995 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif
#include "config.h"
#include "SdText.H"

SdText::SdText()
{
}

SdText::SdText(const Location &loc, Boolean lita)
: lita_(lita)
{
  items_.grow().loc = loc;
  items_.last().index = 0;
}

void SdText::addChar(SyntaxChar c, const Location &loc)
{
  if (items_.length() == 0
      || loc.origin().pointer() != items_.last().loc.origin().pointer()
      || loc.index() != (items_.last().loc.index()
			 + (chars_.length() - items_.last().index))) {
    items_.grow();
    items_.last().loc = loc;
    items_.last().index = chars_.length();
  }
  chars_ += c;
}

void SdText::moveTo(SdText &to)
{
  items_.moveTo(to.items_);
  chars_.moveTo(to.chars_);
  to.lita_ = lita_;
}

Location SdText::endDelimLocation() const
{
  Location loc(items_.last().loc);
  loc += chars_.length() - items_.last().index;
  return loc;
}

SdTextItem::SdTextItem()
{
}

SdTextIter::SdTextIter(const SdText &text)
: ptr_(&text),
  itemIndex_(0)
{
}

Boolean SdTextIter::next(const SyntaxChar *&ptr, size_t &length, Location &loc)
{
  const GrowableVectorD<SdTextItem> &items = ptr_->items_;
  if (itemIndex_ >= items.length())
    return 0;
  loc = items[itemIndex_].loc;
  const String<SyntaxChar> &chars = ptr_->chars_;
  size_t charsIndex = items[itemIndex_].index;
  ptr = chars.pointer() + charsIndex;
  if (itemIndex_ + 1 < items.length())
    length = items[itemIndex_ + 1].index - charsIndex;
  else
    length = chars.length() - charsIndex;
  itemIndex_++;
  return 1;
}
