// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef SgmlParser_INCLUDED
#define SgmlParser_INCLUDED 1

#ifdef __GNUG__
#pragma interface
#endif

#include "CString.H"
#include "ResourcePointer.H"
#include "Location.H"
#include "EntityManager.H"
#include "EntityCatalog.H"

class Event;
class Parser;
class UnivCharsetDesc;
class EventHandler;
struct ParserOptions;
struct ParserSubdocOptions;
class SubdocEntity;
template<class T> class ResourcePointer;
template<class T> class ConstResourcePointer;
class InputSourceOrigin;
class Sd;
class Syntax;

class SgmlParser {
public:
  SgmlParser();			// must call init
  SgmlParser(const CString &document,
	     const ResourcePointer<InputSourceOrigin> &origin,
	     const ConstResourcePointer<EntityManager> &,
	     const ConstResourcePointer<EntityCatalog> &,
	     const UnivCharsetDesc &initialCharset,
	     const ParserOptions &options);
  void init(const CString &document,
	    const ResourcePointer<InputSourceOrigin> &origin,
	    const ConstResourcePointer<EntityManager> &,
	    const ConstResourcePointer<EntityCatalog> &,
	    const UnivCharsetDesc &initialCharset,
	    const ParserOptions &options);
  // for parsing subdocuments
  SgmlParser(const SgmlParser &parent,
	     const SubdocEntity &subdoc,
	     const ResourcePointer<InputSourceOrigin> &origin,
	     const ParserSubdocOptions &options);
  void init(const SgmlParser &parent,
	    const SubdocEntity &subdoc,
	    const ResourcePointer<InputSourceOrigin> &origin,
	    const ParserSubdocOptions &);
  ~SgmlParser();
  Event *nextEvent();
  void parseAll(EventHandler &);
  ConstResourcePointer<Sd> sd() const;
  ConstResourcePointer<Syntax> instanceSyntax() const;
  ConstResourcePointer<Syntax> prologSyntax() const;
  const EntityManager &entityManager() const;
  void addInclude(const CString &);
  void activateLinkType(const CString &);
  void allLinkTypesActivated();
private:
  SgmlParser(const SgmlParser &);
  void operator=(const SgmlParser &);
  Parser *parser_;
};

#endif /* not SgmlParser_INCLUDED */
