// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef ShortReferenceMap_INCLUDED
#define ShortReferenceMap_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

#include "Vector.H"
#include "GrowableVectorD.H"
#include "CString.H"
#include "Named.H"
#include "Boolean.H"
#include "Entity.H"
#include "ResourcePointer.H"

class ShortReferenceMap : public Named {
public:
  ShortReferenceMap();
  ShortReferenceMap(const CString &);
  Boolean defined() const;
  void setNameMap(GrowableVectorD<CString> &map);
  void setEntityMap(Vector<ConstResourcePointer<Entity> > &map);
  Boolean lookup(int i, const CString *&) const;
  const CString *entityName(size_t i) const;
  const ConstResourcePointer<Entity> &entity(size_t i) const;
  Boolean used() const;
  void setUsed();
  const Location &defLocation() const;
  void setDefLocation(const Location &);
private:
  ShortReferenceMap(const ShortReferenceMap &);	// undefined
  void operator=(const ShortReferenceMap &);	// undefined
  GrowableVectorD<CString> nameMap_;
  Vector<ConstResourcePointer<Entity> > entityMap_;
  ConstResourcePointer<Entity> nullEntity_;
  Boolean used_;
  Location defLocation_;
};

inline
Boolean ShortReferenceMap::defined() const
{
  return nameMap_.length() > 0;
}

inline
const CString *ShortReferenceMap::entityName(size_t i) const
{
  if (i < nameMap_.length() && nameMap_[i].length() != 0)
    return &nameMap_[i];
  else
    return 0;
}

inline
const ConstResourcePointer<Entity> &ShortReferenceMap::entity(size_t i) const
{
  if (i < entityMap_.length())
    return entityMap_[i];
  else
    return nullEntity_;
}

inline
void ShortReferenceMap::setEntityMap(Vector<ConstResourcePointer<Entity> >
				     &map)
{
  map.moveTo(entityMap_);
}

inline
Boolean ShortReferenceMap::used() const
{
  return used_;
}

inline
void ShortReferenceMap::setUsed()
{
  used_ = 1;
}

inline
void ShortReferenceMap::setDefLocation(const Location &loc)
{
  defLocation_ = loc;
}

inline
const Location &ShortReferenceMap::defLocation() const
{
  return defLocation_;
}

#endif /* not ShortReferenceMap_INCLUDED */
