// Copyright (c) 1995 James Clark
// See the file COPYING for copying permission.

#include "config.h"
#include "SGMLGenerator.H"
#include "ParserEventGenerator.H"
#include "OutputCharStream.H"
#include "LineOutputCodingSystem.H"
#include "programName.H"
#include "macros.H"
#include "getopt.h"
#include "NormMessages.H"
#include "EntityToolkit.H"

#include "outOfMemory.H"
#include "xnew.h"

#include <stdlib.h>
#include <iostream.h>

inline
void usage(Messenger &mgr)
{
  mgr.message(NormMessages::usageInfo);
}

int main(int argc, char **argv)
{
  set_new_handler(outOfMemory);
#ifdef __GNUG__
  // cout is a performance disaster in libg++ unless we do this.
  ios::sync_with_stdio(0);
#endif
  programName = argv[0];
  int opt;
  char arg[2];
  EntityToolkit kit;
  ParserEventGenerator parser;
  
  opterr = 0;			// no messages from getopt
  // Colon at beginning is Posix.2ism that says to return : rather than ? for
  // missing option argument.
  while ((opt = getopt(argc, argv, ":ek:w:c:d:")) >= 0) {
    switch (opt) {
    case 'c':
      parser.setOption(ParserEventGenerator::addCatalog, optarg);
      break;
    case 'd':
      parser.setOption(ParserEventGenerator::addSearchDir, optarg);
      break;
    case 'e':
      parser.setOption(ParserEventGenerator::showOpenEntities);
      break;
    case 'w':
      parser.setOption(ParserEventGenerator::enableWarning, optarg);
      break;
    case 'k':
      {
	for (const char *s = optarg; *s; s++)
	  switch (*s) {
	  case 'c':
	    parser.setOption(ParserEventGenerator::outputCommentDecls);
	    break;
	  case 'm':
	    parser.setOption(ParserEventGenerator::outputMarkedSections);
	    break;
	  default:
	    arg[0] = *s;
	    arg[1] = '\0';
	    kit.message(NormMessages::invalidKeepLetter,
			StringMessageArg(kit.codingSystem()->convertIn(arg)));
	    break;
	  }
      }
      break;
    case ':':
      arg[0] = optopt;
      arg[1] = '\0';
      kit.message(NormMessages::missingOptionArgError,
		  StringMessageArg(kit.codingSystem()->convertIn(arg)));
      usage(kit);
      return 1;
    case '?':
      arg[0] = optopt;
      arg[1] = '\0';
      kit.message(NormMessages::invalidOptionError,
		  StringMessageArg(kit.codingSystem()->convertIn(arg)));
      usage(kit);
      return 1;
    default:
      CANNOT_HAPPEN();
    }
  }
  parser.init(argc - optind, argv + optind);
  LineOutputCodingSystem line(kit.codingSystem(), '\n', '\r', '\n');
  IosOutputCharStream os(cout.rdbuf(), &line);
  SGMLGenerator sgmlGen(&os);
  parser.run(sgmlGen);
  sgmlGen.end();
  return parser.nErrors() > 0;
}
