// Copyright (c) 1995 James Clark
// See the file COPYING for copying permission.

#ifndef CopyEventHandler_INCLUDED
#define CopyEventHandler_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

#include "Event.H"
#include "OutputCharStream.H"
#include "GrowableVectorD.H"
#include "MarkupEventHandler.H"

// This should be local to CopyEventHandler, but some compilers
// can't handle this.

struct CopyEventHandlerEntityInfo {
  StrOutputCharStream str;
  ConstResourcePointer<EntityOrigin> ref;
};

class CopyEventHandler : public MarkupEventHandler {
public:
  enum NormalizeFlags {
    normalizeOmittag = 01,
    normalizeShortref = 02,
    normalizeNet = 04,
    normalizeEmptytag = 010,
    normalizeUnclosed = 020,
    normalizeAttname = 040,
    normalizeAttvalue = 0100,
    normalizeAttspec = 0200,
    normalizeShorttag = (normalizeNet|normalizeEmptytag|normalizeUnclosed
			 |normalizeAttname|normalizeAttvalue|normalizeAttspec),
    normalizeRank = 0400,
    normalizeOmittagHoist = 01000,
    normalizeExpand = 02000,
    normalizeExpandAll = 04000,
    normalizeCurrent = 010000,
    normalizeLower = 020000,
    normalizeReserved = 040000,
    normalizeExpandProlog = 0100000,
    normalizeIncludeProlog = 0200000,
    normalizeMarkedSection = 0400000
    };
  CopyEventHandler(OutputCharStream *, unsigned flags,
		   const CString &outputEntity);
  void reOrigin(ReOriginEvent *);
  void sSep(SSepEvent *);
  void ignoredRs(IgnoredRsEvent *);
  void data(DataEvent *);
  void startElement(StartElementEvent *);
  void endElement(EndElementEvent *);
  void sdataEntity(SdataEntityEvent *);
  void externalDataEntity(ExternalDataEntityEvent *);
  void subdocEntity(SubdocEntityEvent *);
  void pi(PiEvent *);
  void endProlog(EndPrologEvent *);
  void markedSectionStart(MarkedSectionStartEvent *);
  void markedSectionEnd(MarkedSectionEndEvent *);
  void entityStart(EntityStartEvent *);
  void entityEnd(EntityEndEvent *);
  void ignoredChars(IgnoredCharsEvent *);
  void sgmlDecl(SgmlDeclEvent *);
  void usemap(UsemapEvent *);
  void uselink(UselinkEvent *);
  void startDtd(StartDtdEvent *);
  void startLpd(StartLpdEvent *);
  void endDtd(EndDtdEvent *);
  void endLpd(EndLpdEvent *);
  void entityDecl(EntityDeclEvent *);
  void shortrefDecl(ShortrefDeclEvent *);
  void markup(const Location &loc, const Markup &markup);
private:
  virtual void reportTagInSpecialMarkedSection(const Location &) = 0;
  OutputCharStream &os();
  void startSubset(StartSubsetEvent *);
  void endSubset(MarkupEvent *);
  void literal(const Text &text);
  void attributeSpecList(MarkupIter &, const AttributeList &);
  void unspecifiedAttributeValues(const AttributeList &atts,
				  const CString &beforeFirst);
  void withNamedCharRef(const CString &, const Location &loc);
  void withNamedCharRef(const Char *p, size_t n, const Location &loc);
  void writeData(const Char *p, size_t n, const Location &loc);
  void sdParamLiteral(const SdText &text);
  void entityRef(const EntityOrigin *);
  Boolean doNothing(Event *);
  Boolean noOutput();
  Boolean isSgmlChar(Char);
  const CString &elementTypeOrigName(const ElementType *, CString &buf);
  const CString &generalName(const CString &name, CString &buf);
  const CString &entityName(const CString &name, CString &buf);
  const CString &lowerCaseName(const CString &name, CString &buf);
  void handleChange();
  void outputMarkup(const Location &loc, const Markup &markup);
  OutputCharStream *os_;
  OutputCharStream *topOs_;
  Boolean inInstance_;
  unsigned entityLevel_;
  unsigned outputEntityLevel_;
  ConstResourcePointer<Syntax> syntax_;
  ConstResourcePointer<Syntax> instanceSyntax_;
  ConstResourcePointer<Sd> sd_;
  SubstTable<Char> lowerSubst_;
  unsigned normalizeFlags_;
  CString outputEntity_;
  GrowableVectorD<CString> elementTypeOrigNames_;
  unsigned omittagHoist_;
  ConstResourcePointer<EntityOrigin> entityOrigin_;
  GrowableVectorD<CopyEventHandlerEntityInfo> entityStack_;
  Boolean inSpecialMarkedSection_;
  Boolean hasInternalSubset_;
  Boolean mustOmitEnd_;
  const AttributeList *currentAttributes_;
};

inline
OutputCharStream &CopyEventHandler::os()
{
  return *os_;
}

#endif /* not CopyEventHandler_INCLUDED */
