// Copyright (c) 1995 James Clark
// See the file COPYING for copying permission.

#include "config.h"

#include "EntityToolkit.H"
#include "CodingSystem.H"
#include "InputSource.H"
#include "OutputCharStream.H"
#include "LineOutputCodingSystem.H"
#include "programName.H"
#include "getopt.h"
#include "SpentMessages.H"
#include "MessageArg.H"

#include "outOfMemory.H"
#include "xnew.h"
#include "macros.H"

#include <iostream.h>
#ifdef HAVE_LOCALE
#include <locale.h>
#endif
#include <stdlib.h>

const Char RE = '\r';
const Char RS = '\n';

inline
void usage(Messenger &mgr)
{
  mgr.message(SpentMessages::usageInfo);
}

int main(int argc, char *const *argv)
{
  set_new_handler(outOfMemory);
#ifdef HAVE_LOCALE
  setlocale(LC_ALL, "");
#endif
  programName = argv[0];
#ifdef __GNUG__
  // cout is a performance disaster in libg++ unless we do this.
  ios::sync_with_stdio(0);
#endif
  EntityToolkit kit;
  int opt;
  const OutputCodingSystem *outputCodingSystem = 0;
  Boolean rawOutput = 0;
  char arg[2];
  opterr = 0;			// no messages from getopt
  // Colon at beginning is Posix.2ism that says to return : rather than ? for
  // missing option argument.
  while ((opt = getopt(argc, argv, ":b:rv")) >= 0) {
    switch (opt) {
    case 'b':
      outputCodingSystem = kit.lookupCodingSystem(optarg);
      if (!outputCodingSystem)
	kit.message(SpentMessages::unknownBctf,
		       StringMessageArg(kit.codingSystem()
					->convertIn(optarg)));
      break;
    case 'r':
      rawOutput = 1;
      break;
    case 'v':
      // print the version number
      {
	extern const char *versionString;
	// FIXME perhaps we should have a constant argument instead
	kit.message(SpentMessages::versionInfo,
		    StringMessageArg(kit.codingSystem()
				     ->convertIn(versionString)));
      }
      break;
    case ':':
      arg[0] = optopt;
      arg[1] = '\0';
      kit.message(SpentMessages::missingOptionArgError,
		  StringMessageArg(kit.codingSystem()->convertIn(arg)));
      usage(kit);
      return 1;
    case '?':
      arg[0] = optopt;
      arg[1] = '\0';
      kit.message(SpentMessages::invalidOptionError,
		  StringMessageArg(kit.codingSystem()->convertIn(arg)));
      usage(kit);
      return 1;
    default:
      CANNOT_HAPPEN();
    }
  }
  kit.initEntityManager();
  CString sysid;
  if (!kit.makeSystemId(argc - optind, argv + optind, sysid))
    return 1;
  InputSource *in = kit.open(sysid);
  if (!in)
    return 1;
  if (!outputCodingSystem)
    outputCodingSystem = kit.codingSystem();
  LineOutputCodingSystem line(outputCodingSystem, RS, RE, '\n');
  IosOutputCharStream os(cout.rdbuf(),
			 (rawOutput
			  ? outputCodingSystem
			  : (const OutputCodingSystem *)&line));
  for (;;) {
    Xchar c = in->get(kit);
    if (c == InputSource::eE)
      break;
    in->extendToBufferEnd();
    os.write(in->currentTokenStart(), in->currentTokenLength());
  }
  return 0;
}
