// Copyright (c) 1994 James Clark
// See the file COPYING for copying permission.

#ifndef CodingSystem_INCLUDED
#define CodingSystem_INCLUDED 1

#ifdef __GNUG__
#pragma interface
#endif

#include "types.H"
#include "Boolean.H"
#include "CString.H"

#include <stddef.h>

class Decoder {
public:
  virtual ~Decoder();
  virtual size_t decode(Char *, const char *, size_t, const char **) = 0;
  virtual Boolean convertOffset(unsigned long &offset) const;
};

class streambuf;

class Encoder {
public:
  class Handler {
  public:
    virtual void handleUnencodable(Char, streambuf *) = 0;
  };
  Encoder();
  virtual ~Encoder();
  virtual void output(const Char *, size_t, streambuf *) = 0;
  // This outputs a byte order mark with Unicode.
  virtual void startFile(streambuf *);
  virtual void output(Char *, size_t, streambuf *);
  void setUnencodableHandler(Handler *);
protected:
  void handleUnencodable(Char, streambuf *);
private:
  Handler *unencodableHandler_;
};

class InputCodingSystem {
public:
  virtual ~InputCodingSystem();
  virtual Decoder *makeDecoder() const = 0;
  // must be > 0
  virtual unsigned minBytesPerChar() const;
  CString convertIn(const char *) const;
  virtual Boolean isIdentity() const;
};

class OutputCodingSystem {
public:
  virtual ~OutputCodingSystem();
  virtual Encoder *makeEncoder() const = 0;
  virtual unsigned fixedBytesPerChar() const;
  String<char> convertOut(const CString &) const;
};

class CodingSystem : public InputCodingSystem, public OutputCodingSystem {
};

inline
void Encoder::handleUnencodable(Char c, streambuf *sbufp)
{
  if (unencodableHandler_)
    unencodableHandler_->handleUnencodable(c, sbufp);
}

inline
void Encoder::setUnencodableHandler(Handler *handler)
{
  unencodableHandler_ = handler;
}

#endif /* not CodingSystem_INCLUDED */
