// Copyright (c) 1994, 1995 James Clark
// See the file COPYING for copying permission.

#ifndef ExtendEntityManager_INCLUDED
#define ExtendEntityManager_INCLUDED 1

#ifdef __GNUG__
#pragma interface
#endif

#include "EntityManager.H"
#include "CharsetInfo.H"
#include "types.H"
#include "Boolean.H"
#include "CString.H"
#include "types.H"
#include "CopyGrowableVectorD.H"
#include "Location.H"

class StorageManager;
class InputCodingSystem;
class Messenger;

struct StorageObjectSpec {
  StorageObjectSpec();
  StorageManager *storageManager;
  const char *codingSystemName;
  const InputCodingSystem *codingSystem;
  CString specId;		// specified id
  CString baseId;		// id that specified id is relative to
  CString id;			// actual id used (filled in after opening)
  enum Records {
    find,
    cr,
    lf,
    crlf,
    asis
    };
  Records records;
  PackedBoolean notrack;
  PackedBoolean zapEof;		// zap a final Ctrl-Z
};

struct ParsedSystemId : public CopyGrowableVectorD<StorageObjectSpec> {
  ParsedSystemId();
  void unparse(const CharsetInfo &charset, CString &result) const;
  String<char> canonId() const;
};

struct StorageObjectLocation {
  const StorageObjectSpec *storageObjectSpec;
  unsigned long lineNumber;
  unsigned long columnNumber;
  unsigned long byteIndex;
  unsigned long storageObjectOffset;
};

class ExtendEntityManager : public EntityManager {
public:
  virtual void registerStorageManager(StorageManager *) = 0;
  virtual void registerCodingSystem(const char *, const InputCodingSystem *)
    = 0;
  virtual InputSource *openIfExists(const CString &sysid,
				    const CharsetInfo &,
				    InputSourceOrigin *,
				    Boolean mayRewind,
				    Messenger &) const = 0;
  virtual Boolean expandSystemId(const CString &,
				 const Location &,
				 Boolean isNdata,
				 const CharsetInfo &,
				 Messenger &,
				 CString &) const = 0;
  virtual Boolean mergeSystemIds(const CString *nSysids,
				 size_t n,
				 const CharsetInfo &,
				 Messenger &mgr,
				 CString &) const = 0;
  virtual Boolean parseSystemId(const CString &str,
				const CharsetInfo &idCharset,
				Boolean isNdata,
				const StorageObjectSpec *defSpec,
				Messenger &mgr,
				ParsedSystemId &parsedSysid) const = 0;
  static Boolean externalize(const ExternalInfo *,
			     Offset,
			     StorageObjectLocation &);
  static const ParsedSystemId *
    externalInfoParsedSystemId(const ExternalInfo *);
  static ExtendEntityManager *make(StorageManager *,
				   const InputCodingSystem *);
};

#endif /* not ExtendEntityManager_INCLUDED */
