// Copyright (c) 1994, 1995 James Clark
// See the file COPYING for copying permission.

#ifndef PosixStorage_INCLUDED
#define PosixStorage_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

#include "StorageManager.H"
#include "DescriptorManager.H"
#include "GrowableVectorD.H"
#include "CString.H"
#include "CharsetInfo.H"

class Messenger;
class CharsetInfo;
class UnivCharsetDesc;
class Filename;
class OutputCodingSystem;

class PosixStorageManager : public StorageManager {
public:
  PosixStorageManager(const char *type,
		      const UnivCharsetDesc &filenameCharset,
		      const OutputCodingSystem *filenameCodingSystem,
		      int maxFDs);
  StorageObject *makeStorageObject(const CString &id,
				   const CString &baseId,
				   Boolean mayRewind,
				   Messenger &,
				   CString &foundId);
  Boolean resolveRelative(const CString &, CString &, Boolean syntactic = 0)
    const;
  String<char> canonId(const CString &);
  const char *type() const;
  void addSearchDir(const CString &);
  Boolean transformNeutral(CString &, Boolean fold, Messenger &) const;
private:
  Boolean isAbsolute(const CString &) const;
  CString extractDir(const CString &) const;
  CString combineDir(const CString &, const CString &) const;
  PosixStorageManager(const PosixStorageManager &); // undefined
  void operator=(const PosixStorageManager &);	    // undefined
  DescriptorManager descriptorManager_;
  const OutputCodingSystem *filenameCodingSystem_;
  const char *type_;
  GrowableVectorD<CString> searchDirs_;
};

class PosixFdStorageManager : public StorageManager {
public:
  PosixFdStorageManager(const char *type,
			const UnivCharsetDesc &filenameCharset);
  StorageObject *makeStorageObject(const CString &id,
				   const CString &baseId,
				   Boolean mayRewind,
				   Messenger &mgr,
				   CString &foundId);
  const char *type() const;
  Boolean inheritable() const;
private:
  PosixFdStorageManager(const PosixFdStorageManager &); // undefined
  void operator=(const PosixFdStorageManager &);	    // undefined
  const char *type_;

};

#endif /* not PosixStorage_INCLUDED */
