// Copyright (c) 1994, 1995 James Clark
// See the file COPYING for copying permission.

#ifdef __GNUG__
#pragma implementation
#endif

#include "config.h"
#include "RewindStorageObject.H"
#include "macros.H"

RewindStorageObject::RewindStorageObject(Boolean mayRewind, Boolean canSeek)
: mayRewind_(mayRewind), canSeek_(canSeek),
  savingBytes_(mayRewind && canSeek), readingSaved_(0)
{
}

Boolean RewindStorageObject::rewind(Messenger &mgr)
{
  ASSERT(mayRewind_);
  if (canSeek_)
    return seekToStart(mgr);
  else {
    readingSaved_ = 1;
    nBytesRead_ = 0;
    return 1;
  }
}

void RewindStorageObject::unread(const char *s, size_t n)
{
  savedBytes_.append(s, n);
  if (!readingSaved_) {
    readingSaved_ = 1;
    nBytesRead_ = 0;
  }
}

void RewindStorageObject::willNotRewind()
{
  mayRewind_ = 0;
  savingBytes_ = 0;
  if (!readingSaved_)
    savedBytes_.clear();
}

Boolean RewindStorageObject::readSaved(char *buf, size_t bufSize,
				       size_t &nread)
{
  if (!readingSaved_)
    return 0;
  if (nBytesRead_ >= savedBytes_.length()) {
    if (!mayRewind_)
      savedBytes_.clear();
    readingSaved_ = 0;
    return 0;
  }
  nread = savedBytes_.length() - nBytesRead_;
  if (nread > bufSize)
    nread = bufSize;
  memcpy(buf, savedBytes_.pointer() + nBytesRead_, nread);
  nBytesRead_ += nread;
  return 1;
}
