// Copyright (c) 1994, 1995 James Clark
// See the file COPYING for copying permission.

#ifndef StorageManager_INCLUDED
#define StorageManager_INCLUDED 1
#ifdef __GNUG__
#pragma interface
#endif

class DescriptorManager;
class Messenger;

#include "CString.H"
#include "types.H"
#include "CharsetInfo.H"
#include <stddef.h>

class StorageManager;
class CharsetInfo;
class Messenger;

class StorageObject {
public:
  StorageObject();
  virtual ~StorageObject();
  virtual Boolean read(char *buf, size_t bufSize, Messenger &,
		       size_t &nread) = 0;
  virtual Boolean rewind(Messenger &) = 0;
  virtual void willNotRewind();
  virtual size_t getBlockSize() const;
private:
  StorageObject(const StorageObject &);	// undefined
  void operator=(const StorageObject &); // undefined
};

class StorageManager {
public:
  StorageManager(const UnivCharsetDesc &idCharset);
  virtual StorageObject *makeStorageObject(const CString &specId,
					   const CString &baseId,
					   Boolean mayRewind,
					   Messenger &mgr,
					   CString &actualId) = 0;
  virtual String<char> canonId(const CString &);
  virtual const char *type() const = 0;
  virtual Boolean inheritable() const;
  virtual Boolean transformNeutral(CString &, Boolean fold, Messenger &) const;
  // Resolve a possibly relative ID by examining the base and specified IDs.
  // Put the resolved ID in specID.
  // Return 0 if it cannot be resolved yet becase the specified ID is relative
  // and physical searching is required to resolve it; in this case the
  // base will be passed to makeStorageObject.
  // Otherwise return 1; in this case the base will be discarded, and the
  // resolved ID will be passed to makeStorageObject.
  virtual Boolean resolveRelative(const CString &base,
				  CString &specId,
				  Boolean syntactic) const;
  virtual Boolean guessIsId(const CString &, const CharsetInfo &) const;
  virtual ~StorageManager();
  const CharsetInfo &idCharset() const;
  const CString &reString() const;
protected:
  CString reString_;
private:
  StorageManager(const StorageManager &); // undefined
  void operator=(const StorageManager &); // undefined
  CharsetInfo idCharset_;
};

inline
const CharsetInfo &StorageManager::idCharset() const
{
  return idCharset_;
}

inline
const CString &StorageManager::reString() const
{
  return reString_;
}

#endif /* not StorageManager_INCLUDED */
